@@ -1,24 +1,47 @@
 import pandas as pd
 from sqlalchemy import create_engine
-from dask import delayed
 
 class da_tran_SQL :
-    """interact with SQL"""
-    def __init__(self, sql_type, host_name, database_name, user, password 
+    """interact with SQL
+
+    :sql_type = string - type of database, support MSSQL, MYSQL, POSTGRESQL, SQLITE, BIGQUERY
+
+    :host_name = string 
+       - for MSSQL, MYSQL, POSTGRESQL : database's hostname or ip address
+       - for SQLITE : location of db file
+       - for BIGQUERY : project id on GCP
+
+    : database_name = string
+       - for MSSQL, MYSQL, POSTGRESQL : database's name
+       - for SQLITE : Leave as ''
+       - for BIGQUERY : dataset's name
+
+    : user = string - User for MSSQL, MYSQL, POSTGRESQL
+
+    : password = string - password for MSSQL, MYSQL, POSTGRESQL
+
+    : credentials_path = string - path to GCP credentials file (json)
+
+    more sample of this class at https://github.com/chanon-kr/Shared_Function/blob/main/samples/database.ipynb
+    """
+    def __init__(self, sql_type, host_name, database_name, user = '', password = '' , credentials_path = None
                 , chunksize = 150, partition_size = 5000, parallel_dump = False, max_parallel = 2, **kwargs):
         """Create connection to SQL Server"""
         sql_type = sql_type.upper()
         self.sql_type = sql_type
         self.chunksize = int(chunksize)
+        if sql_type == 'BIGQUERY' : self.method = None
+        else : self.method = 'multi'
         self.partition_size = int(partition_size)
         self.parallel_dump = parallel_dump
+        if parallel_dump : print('Set to use parallel dump, there is a risk to use this method, use it with caution')
         self.max_parallel = int(max_parallel)
-
         type_dic = {
                     'MSSQL' : ['mssql', 'pymssql', '1433','[',']','EXEC'],
-                    'MYSQL' : ['mysql', 'pymysql','3306','','','CALL'],
+                    'MYSQL' : ['mysql', 'pymysql','3306','`','`','CALL'],
                     'POSTGRESQL' : ['postgresql', 'psycopg2','5432','"','"','SELECT * FROM '],
-                    'SQLITE' : ['sqlite', 'sqlite', '','[',']','EXEC']
+                    'SQLITE' : ['sqlite', 'sqlite', '','[',']','EXEC'],
+                    'BIGQUERY' : ['bigquery', 'bigquery','','`','`','CALL'],
                     }
         
         if (sql_type == 'SQLITE') & (chunksize == 150) :
@@ -34,27 +57,50 @@ class da_tran_SQL :
         else : additional_param = '?' + kwargs['parameter']
 
         self.begin_name, self.end_name, self.call_SP = type_dic[sql_type][3], type_dic[sql_type][4], type_dic[sql_type][5]
-
-        if (sql_type == 'SQLITE') & (kwargs.get('driver',None) == None) :
-            connection_str = str(type_dic[sql_type][0] + ':///' + host_name + additional_param)
-        else :
-            connection_str = str(type_dic[sql_type][0] + '+' + kwargs.get('driver',type_dic[sql_type][1])
-                            + '://' + user + ':' + password + '@' + host_name
-                            + ':' + kwargs.get('port',type_dic[sql_type][2]) + '/' + database_name + additional_param)
         
-        self.engine = create_engine(connection_str)
-        print(pd.read_sql_query("""SELECT 'Connection OK'""", con = self.engine).iloc[0,0]) 
+        if (sql_type == 'SQLITE') & (kwargs.get('driver',None) == None) :
+            connection_str = """{}:///{}{}""".format(type_dic[sql_type][0] , host_name , additional_param)
+        elif sql_type == 'BIGQUERY' :
+            connection_str = "{}://{}/{}".format(type_dic[sql_type][0],host_name,database_name)
+        else :
+            connection_str = """{}+{}://{}:{}@{}:{}/{}{}""".format(type_dic[sql_type][0],kwargs.get('driver',type_dic[sql_type][1])
+                                                            ,user,password,host_name,kwargs.get('port',type_dic[sql_type][2])
+                                                            ,database_name , additional_param)
+        
+        if (credentials_path != '') & (sql_type == 'BIGQUERY') :
+            # self.engine = create_engine(connection_str, credentials_path = credentials_path)
+            from google.oauth2 import service_account
+            self.project_id = host_name
+            # self.dataset = self.begin_name + database_name + self.end_name + '.'
+            self.dataset = database_name + '.'
+            self.engine = create_engine(connection_str)
+            if credentials_path == None : self.credentials = None
+            else : self.credentials = service_account.Credentials.from_service_account_file(credentials_path)
+            print(pd.read_gbq("""SELECT 'Connection OK'""",project_id = self.project_id,credentials = self.credentials).iloc[0,0]) 
+        else :
+            self.credentials = ''
+            self.dataset = ''
+            self.engine = create_engine(connection_str)
+            print(pd.read_sql_query("""SELECT 'Connection OK'""", con = self.engine).iloc[0,0]) 
 
     def sub_dump(self, df_in,table_name_in,mode_in) :
         """normal to_sql for dask's delayed in dump_main"""
-        df_in.to_sql(table_name_in, con = self.engine, index = False,if_exists = mode_in,chunksize = self.chunksize, method = 'multi')
+        if (self.credentials != '') & (self.sql_type == 'BIGQUERY') :
+            # print('{}{}'.format(self.dataset , table_name_in))
+            df_in.to_gbq('{}{}'.format(self.dataset , table_name_in),project_id = self.project_id
+                                        ,credentials = self.credentials , reauth = True, if_exists = mode_in)
+        else :
+            df_in.to_sql(table_name_in, con = self.engine, index = False,if_exists = mode_in,chunksize = self.chunksize, method = self.method)
         return len(df_in)
 
     def dump_main(self, df_in, table_name_in ,mode_in) :
         """Divide and Dump Dataframe"""
         if len(df_in) <= self.partition_size : sum_len = self.sub_dump(df_in,table_name_in,mode_in) 
         else :
-            if self.parallel_dump : dask_dump = delayed(self.sub_dump)
+            if self.parallel_dump : 
+                print('About to use parallel dump, there is a risk to use this method, use it with caution')
+                from dask import delayed
+                dask_dump = delayed(self.sub_dump)
             i, j, sum_len, df_length = 0, 1, 0, len(df_in)
             while i < df_length :
                 i_next = i + self.partition_size
@@ -70,18 +116,18 @@ class da_tran_SQL :
         return sum_len
 
 
-    def read(self, table_name_in, condition_in = '', SP = False, param = ''):
+    def read(self, table_name_in, condition_in = '', SP = False,raw = False, param = ''):
         """Read Table or View or Store Procedure"""
         if SP :
             if self.sql_type == 'POSTGRESQL' : return 'SP/Function None Avaliable for PostgreSQL, right now'
-            sql_q = self.call_SP + ' ' + table_name_in + ' '
+            sql_q = self.call_SP + ' ' + self.dataset + table_name_in + ' '
             if param == '' : raise Exception("Please insert SP's parameter")
             elif type(param) != dict : raise Exception("param must be dict")
             else :
                 n = 0
                 for i in param.keys() :
                     if type(param[i]) == str : param[i] = """'{}'""".format(param[i])
-                    if self.sql_type in ['MYSQL','POSTGRESQL'] : 
+                    if self.sql_type in ['MYSQL','POSTGRESQL','BIGQUERY'] : 
                         if n == 0 : sql_q += ' (' 
                         else : sql_q += ' , '
                         sql_q += """ {}""".format(param[i])
@@ -93,11 +139,15 @@ class da_tran_SQL :
                         if n > 0 : sql_q += ' , '
                         sql_q += """ {} = {}""".format(i, param[i])
                     n += 1
+        elif raw : sql_q = str(table_name_in)
         else :
-            sql_q = """SELECT * FROM {}{}{}""".format(self.begin_name,table_name_in,self.end_name)
+            sql_q = """SELECT * FROM {}{}{}{}""".format(self.dataset,self.begin_name,table_name_in,self.end_name)
             if not condition_in == '' :
                 sql_q += """ WHERE """ + condition_in
-        return pd.read_sql_query(sql_q , con = self.engine)
+        if (self.credentials != '') & (self.sql_type == 'BIGQUERY') :
+            return pd.read_gbq(sql_q,project_id = self.project_id,credentials = self.credentials)
+        else :
+            return pd.read_sql_query(sql_q , con = self.engine)
 
     def write_logic_sql(self, key , value_in):
         """Write SQL Condition Query that include >, <, ="""
@@ -137,10 +187,16 @@ class da_tran_SQL :
                 logic_query = '({}{}{} IS NULL'.format(self.begin_name , key, self.end_name) + ')'
             else :
                 filter_filter = str(filter_filter).replace(',)',')')  # tuple with 1 value will be ( x , ) => need to convert
-                logic_query = self.begin_name + key  + self.end_name + ' in ' + filter_filter
+                if self.sql_type != 'BIGQUERY':
+                    logic_query = self.begin_name + key  + self.end_name + ' in ' + filter_filter
+                else :
+                    logic_query = 'CAST(' + self.begin_name + key  + self.end_name + ' AS STRING) in ' + filter_filter
         elif len(filter_filter) > 1 :
             filter_filter = str(filter_filter) # tuple with > 1 values will be ( x, y, z) which can be use in SQL
-            logic_query = self.begin_name + key  + self.end_name + ' in ' + filter_filter
+            if self.sql_type != 'BIGQUERY':
+                logic_query = self.begin_name + key  + self.end_name + ' in ' + filter_filter
+            else :
+                logic_query = 'CAST(' + self.begin_name + key  + self.end_name + ' AS STRING) in ' + filter_filter
         else : logic_query = '' # Return Nothing
         #print(logic_query)
         if "'Will BE rEpLaCe wItH NULL'" in logic_query :
@@ -211,15 +267,9 @@ class da_tran_SQL :
         if table_name_in in list(self.engine.table_names()) :
             print('Start Filter Existing data from df at ',pd.Timestamp.now())
             #for single key
-            if 'str' in str(type(list_key))  :
-                if not list_key in df_in.columns :
-                    raise Exception("{} is not in df's columns".format(list_key))       
-                sql_q = 'select distinct '+ self.begin_name +  list_key + self.end_name +' from ' + table_name_in
-                filter_filter = pd.read_sql_query(sql_q, con = self.engine).iloc[:,0]
-                logic_filter = ~df_in[list_key].isin(filter_filter)
-
-            #for multi keys    
-            elif 'list' in str(type(list_key)) :
+            if 'str' in str(type(list_key))  :  list_key = [list_key]
+                
+            if 'list' in str(type(list_key)) :
                 if not min(x in df_in.columns for x in list_key) :
                     raise Exception("Some keys are not in df's columns")
                 i = 0
@@ -231,9 +281,13 @@ class da_tran_SQL :
                     df_in['key_sql_filter'] = df_in['key_sql_filter'].astype('str') + df_in[filter_name].astype('str')
                     sql_q += self.begin_name +  filter_name + self.end_name + ' ' 
                     i += 1
-                sql_q += ' FROM ' + table_name_in
+                sql_q += ' FROM ' + self.dataset + table_name_in
                 if debug : print(sql_q)
-                filter_filter = pd.read_sql_query(sql_q, con = self.engine)
+                if (self.credentials != '') & (self.sql_type == 'BIGQUERY') :
+                    filter_filter = pd.read_gbq(sql_q,project_id = self.project_id,credentials = self.credentials)
+                else :
+                    filter_filter = pd.read_sql_query(sql_q, con = self.engine)
+                # filter_filter = pd.read_sql_query(sql_q, con = self.engine)
                 filter_filter['key_sql_filter'] = ''
                 for i in filter_filter.columns :
                     if  i == 'key_sql_filter' : pass
