@@ -7,7 +7,7 @@ from email.mime.text import MIMEText
 from email.mime.image import MIMEImage
 import mimetypes
 from datetime import datetime
-import os , csv , re, socket
+import os , csv , re, socket, requests
 
 class email_sender :
     def __init__(self, user, password, servername):
@@ -75,16 +75,43 @@ class email_sender :
         #Sending Email and return status text
         return self.executor(recipients, message)
 
+class lazy_LINE :
+    def __init__(self, token) :
+        self.token = token
+    
+    def help(self) :
+        helper = 'Main Page : https://notify-bot.line.me/en\n'
+        helper += 'Doc : https://notify-bot.line.me/doc/en\n'
+        helper += 'Sticker List : https://developers.line.biz/en/docs/messaging-api/sticker-list'
+        print(helper)
+    
+    def send(self, message , stickerPackageId = '', stickerId = '', notification = True, picture = '') :
+        payload = {}
+        payload['message'] = message
+        payload['notificationDisabled'] = not notification
+
+        if (stickerPackageId != '') & (stickerId != '') :
+            payload['stickerPackageId'] = stickerPackageId
+            payload['stickerId'] = stickerId
+        
+        if picture == '' : 
+            r = requests.post('https://notify-api.line.me/api/notify'
+                                , headers={'Authorization' : 'Bearer {}'.format(self.token)}
+                                , params = payload)
+        else :
+            r = requests.post('https://notify-api.line.me/api/notify'
+                                , headers={'Authorization' : 'Bearer {}'.format(self.token)}
+                                , params = payload , files = {'imageFile': open(picture, 'rb')})
+        return r
 
 def log_csv(file_name,msg_in):
     logic = True
     if os.path.exists(file_name) : logic = False
     if '/' in file_name : 
         if file_name.split('/')[0] != '' :
-            print(file_name.split('/')[0])
             dir_name = re.sub('(/.*\..*)','',file_name)
             if not os.path.isdir(dir_name) : os.mkdir(dir_name)
-    with open(file_name.replace('/',''), 'a', newline='') as csvfile:
+    with open(file_name, 'a', newline='') as csvfile:
         fieldnames = ['Timestamp','msg']
         writer = csv.DictWriter(csvfile, fieldnames=fieldnames)
         if logic : writer.writeheader()
